import pytest
from mock_alchemy.mocking import UnifiedAlchemyMagicMock
from pyramid.config import Configurator
from pyramid.testing import DummyRequest, setUp, tearDown
from unittest.mock import MagicMock, patch

from unittest import mock

from workspaces.notification.schema import NotificationTemplate
from workspaces.notification.services.notification_info import NotificationInfo
from workspaces.notification.services.notification_service import NotificationService

@pytest.fixture(scope="session")
def mock_notification_info() -> NotificationInfo:
    """
    Pytest fixture providing a mock NotificationInfo object

    :return: Mocked NotificationInfo
    """
    email_notification = NotificationTemplate(
        name="email",
        description="A generic email template, requiring subject and message.",
        content="Subject: {{subject}}\n\n{{message}}",
    )

    mock_session = UnifiedAlchemyMagicMock()
    mock_notification_info = NotificationInfo(mock_session)
    mock_session.add(email_notification)
    with patch("workspaces.notification.services.notification_info.NotificationInfo.save_entity"):
        return mock_notification_info


@pytest.fixture(scope="session")
def mock_notification_service(mock_notification_info: NotificationInfo) -> NotificationService:
    """
    Pytest fixture providing a mock NotificationService object

    :return: Mocked NotificationService
    """
    return NotificationService(mock_notification_info)

@pytest.fixture(scope="module")
def test_config() -> Configurator:
    """
    Returns a dummy Configurator object for testing
    :return: Dummy Configurator
    """
    request = DummyRequest()
    config = setUp(request=request)
    config.add_request_method(
        lambda lookup: MagicMock(),
        "notification_info",
        reify=True
    )
    yield config
    tearDown()


@pytest.fixture("module")
def dummy_send(mock_notification_info, mock_notification_service) -> DummyRequest:
    """
    returns a dummy request with a mocked notification info

    :return: DummyRequest with mock NotificationInfo
    """
    request = DummyRequest(
        notification_info=mock_notification_info,
        notification_service=mock_notification_service
    )
    return request
